@@ -8,8 +8,8 @@ import queue
 #sys.path.append('/home/pi/gxf/python/spotii')
 #sys.path.append('/home/pi/gxf/python/spotii/test_chip_handler')
 
-from .send_result import SendResult
-from spotii.define import *
+from communication.send_result import SendResult
+from define import *
 
 
 
