@@ -10,25 +10,14 @@ if __name__ == "__main__":
     parentdir = os.path.dirname(currentdir)
     sys.path.insert(0, parentdir)
     
+from non_block_queue import NonBlockQue
+from main_paras import getDetectionMode, getOperation, setOperation, queueForGui
 from test_handler.i2c_lib import i2c_device
 from test_handler.take_photo import takePhoto, TakePhotoProcedure
 from define import *
+import main_paras
 
 
-# from smbus import SMBus
-# import sys
-# sys.path.append('/home/pi/gxf/python/spotii')
-# sys.path.append('/home/pi/gxf/python/spotii/test_chip_handler')
-# import subprocess
-# from mythread import MyThread
-# import time
-# import queue
-# import threading
-# from i2c_lib import i2c_device
-# from take_photo import takePhoto
-# from datetime import datetime,timezone
-# from define import *
-
 class CassettePolling(threading.Thread):
     def __init__(self, slotIndex, i2c, camera, qResult, qCom, deviceQue):
         threading.Thread.__init__(self)
@@ -45,14 +34,30 @@ class CassettePolling(threading.Thread):
         
         self.takePhotoProcedure = None
         self.running = True
-        
+        self.qrcode_que = NonBlockQue()
+        self.qrPollingStamp=0
+        self.manual_sequence = MANUAL_OPERATION_START
+        self.prevMode=None
+        self.prevCassetteStatus=None
     def stopTakePhotoProcedure(self):
-#        print(self.takePhotoProcedure)
         if self.takePhotoProcedure != None and self.takePhotoProcedure.is_alive():
-            #print("stop take photo procedure")
+            print("stop take photo procedure")
             self.stopTakingPhoto.set()
             self.takePhotoProcedure.join()
             self.stopTakingPhoto.clear()
+            
+    def startTakePhotoProcedure(self,qr_code, responeQue):
+        try:
+            print('start taking photo procedure', qr_code)
+            if qr_code != None:
+                item=[self.slotIndex, DEVICE_STATE_TAKING_PHOTO, qr_code, 'Taking photo']
+                print(item)
+                self.notify(item)
+            self.stopTakePhotoProcedure()
+            self.takePhotoProcedure = TakePhotoProcedure(self.slotIndex, qr_code, self.camera, responeQue, self.stopTakingPhoto)
+            self.takePhotoProcedure.start()
+        except Exception as e:
+            print(e)
         
     def run(self):
         self.periodPoll()
@@ -69,17 +74,98 @@ class CassettePolling(threading.Thread):
 #        self.cameraTest(self.index, self.i2c, self.camera)
 
     def notify(self, item):
-        
+#         if self.slotIndex == 0:
+#             print('deviceStatus',self.deviceStatus, 'new',item[1])
         if item == CLOSE_NOW:
             self.running=False
             self.deviceQue.put(PHOTO_TAKING_STOP)
+            
         elif self.deviceStatus != item[1]:
             self.deviceStatus = item[1]
             if item[1] != DEVICE_STATE_CASSETTE_POLLED:
                 #print('put in queque', item)
                 self.qResult.put(item)
 
-    def getQr(self):
+    def cassetteOut(self):
+        for i in range(5):
+            i2cInstant=i2c_device(self.i2c,I2C_PORT)
+            status=i2cInstant.read_data(I2C_MEM_STATUS)|0x80 #There should be bug on daughter board I2c function. sometimes missed bit 7.
+            
+            if status & CASSETTE_STATUS_FLAG == 0:
+                print(time.strftime('%Y%m%d%H%M%S'), "Wrong flag, re-read.", self.slotIndex, hex(status))
+                time.sleep(2)
+                continue
+            
+            elif status == DEVICE_STATE_CASSETTE_EMPTY:
+                break
+            else:
+                break
+        rtn = False
+#         if(self.slotIndex == 0):
+#             print(self.prevCassetteStatus, status)
+        if self.prevCassetteStatus != status:
+            self.prevCassetteStatus=status
+            if status == DEVICE_STATE_CASSETTE_EMPTY:
+                rtn = True
+            
+        return rtn
+
+    def manualQr(self):
+        status = DEVICE_STATE_CASSETTE_QR
+        qr = None
+#         if self.slotIndex ==0:
+#             print('slot', self.slotIndex, 'Operation',self.manual_sequence)
+            
+        if self.cassetteOut() and self.manual_sequence != MANUAL_OPERATION_TEST:
+            setOperation(self.slotIndex, MANUAL_OPERATION_START)
+#            queueForGui.put([self.slotIndex, DEVICE_STATE_MANUAL_SCAN, '', 'cassette out'])
+#            return status,qr
+        
+        self.manual_sequence = getOperation(self.slotIndex)
+        
+        if self.manual_sequence  == MANUAL_OPERATION_START:
+            self.stopTakePhotoProcedure()
+            self.notify([self.slotIndex, DEVICE_STATE_MANUAL_SCAN, '', 'manual scan'])
+            setOperation(self.slotIndex, MANUAL_OPERATION_START_WAITING)
+        if self.manual_sequence  == MANUAL_OPERATION_START_WAITING:
+            pass
+        elif self.manual_sequence  == MANUAL_OPERATION_SCAN:
+            self.startTakePhotoProcedure(None,self.qrcode_que)
+            self.qrPollingStamp=time.time()
+            setOperation(self.slotIndex, MANUAL_OPERATION_GETTING_QR)
+        elif self.manual_sequence  == MANUAL_OPERATION_GETTING_QR:
+            qr = self.qrcode_que.get()
+            if qr!=None:
+                if qr != NON_QR:
+                    print('got qr from image',qr)
+                    self.qrCode = qr
+                    setOperation(self.slotIndex, MANUAL_OPERATION_TEST)
+                    self.notify([self.slotIndex, DEVICE_STATE_MANUAL_FLIP, self.qrCode, 'Waiting test'])
+                else:
+                    self.notify([self.slotIndex, DEVICE_STATE_MANUAL_INVALID_QR, '', 'invalid qr'])
+                    setOperation(self.slotIndex, MANUAL_OPERATION_RESTART)
+                    
+            elif (time.time()- self.qrPollingStamp) > QR_POLLING_TIMEOUT:
+                print('getting qr time out')
+                self.notify([self.slotIndex, DEVICE_STATE_MANUAL_INVALID_QR, '', 'scan timeout'])
+                setOperation(self.slotIndex, MANUAL_OPERATION_RESTART)
+            else:
+                print('reading qr...')
+        elif self.manual_sequence  == MANUAL_OPERATION_TEST:
+            pass
+        elif self.manual_sequence  == MANUAL_OPERATION_START_TESTING:
+            status = DEVICE_STATE_CASSETTE_VALID
+            qr = self.qrCode
+            setOperation(self.slotIndex, MANUAL_OPERATION_START_WAITING_RESULT)  ##waiting for result
+        elif self.manual_sequence  == MANUAL_OPERATION_START_WAITING_RESULT:
+            pass
+        elif self.manual_sequence == MANUAL_OPERATION_RESTART:
+            pass
+        #print('manual qr')
+        return status, qr
+        
+
+    def autoQr(self):
         try:
             qrCode=''
             status=DEVICE_STATE_CASSETTE_EMPTY
@@ -114,14 +200,18 @@ class CassettePolling(threading.Thread):
             return status,qrCode
         except Exception as err:
             print("CassettePollingException: ", err)
-            return DEVICE_STATE_SUBSYS_EXCEPTION, None        
-
-#     def getPicture_new(self, qr_code):
-#         print("slot",self.slotIndex,qr_code)
-#         self.notify([self.slotIndex, DEVICE_STATE_TAKING_PHOTO, '', 'Taking photo'])
-#         photoFile=takePhoto(self.camera, qr_code+'_'+str(self.slotIndex))
-#         if(photoFile !=None ):
-#             self.qCom.put(photoFile)
+            return DEVICE_STATE_SUBSYS_EXCEPTION, None
+        
+    def getQr(self):
+        currentMode=getDetectionMode()
+        if self.prevMode!=None and self.prevMode!=currentMode:
+            print('mode changed')
+            self.stopTakePhotoProcedure()
+        self.prevMode=currentMode
+        if currentMode == CASSETTE_DETECTION_MODE_AUTO:
+            return self.autoQr()
+        return self.manualQr()
+    
 
 ##self.notifyQue.put([self.slotNo, int(parsing[RSLT][RCODE]), self.cassetteId, parsing[RSLT][MSSG], recordId[self.slotNo]])
     def deviceDeal(self):  # return [device_slotIndex, state, QRcode, message]
@@ -134,18 +224,21 @@ class CassettePolling(threading.Thread):
         status,qr_code =self.getQr();
         if(status == DEVICE_STATE_CASSETTE_POLLED):
             self.notify( [self.slotIndex, DEVICE_STATE_CASSETTE_POLLED, '', 'no change'])
-            return
         elif(status == DEVICE_STATE_CASSETTE_VALID):
-            
-            item=[self.slotIndex, DEVICE_STATE_TAKING_PHOTO, qr_code, 'Taking photo']
-            print(item)
-            self.notify(item)
-            self.stopTakePhotoProcedure()
-            self.takePhotoProcedure = TakePhotoProcedure(self.slotIndex, qr_code, self.camera, self.qCom, self.stopTakingPhoto)
-            self.takePhotoProcedure.start()
+            if main_paras.sign_in_token == '':
+                main_paras.queueForGui.put([SIGN_IN_FIRST_INDEX,'','',''])
+            else:
+                self.startTakePhotoProcedure(qr_code,self.qCom)
+        elif(status == DEVICE_STATE_CASSETTE_QR):
+#             if self.slotIndex == 0:
+#                 print('manual waiting qr')
+            pass            
         else:
-            self.stopTakePhotoProcedure()
-            self.notify([self.slotIndex, status, '','Cassette error'])
+            if main_paras.sign_in_token == '' and status != DEVICE_STATE_CASSETTE_EMPTY:
+                main_paras.queueForGui.put([SIGN_IN_FIRST_INDEX,'','',''])
+            else:
+                self.stopTakePhotoProcedure()
+                self.notify([self.slotIndex, status, '','Cassette error'])
             
     def periodPoll(self):
         #if self.slotIndex == 0:
@@ -159,10 +252,12 @@ class CassettePolling(threading.Thread):
         
     def cameraTest(self):
         photoFile=takePhoto(self.camera, "cameraTest"+'_'+str(self.slotIndex))
-
-
+        
 if __name__ == "__main__":
     import queue
+    from main_paras import setDetectionMode
+    
+    setDetectionMode(CASSETTE_DETECTION_MODE_MANUAL)
 
     qForGui=queue.Queue()
     qForCom=queue.Queue()
